/******************************************************************************
 * Project:  OpenGIS Simple Features Reference Implementation
 * Purpose:  Private definitions for OGR DXF Driver.
 * Author:   Bishop (aka Barishnikov Dmitry), polimax@mail.ru
 *
 ******************************************************************************
 * Copyright (c) 2008, Bishop <polimax@mail.ru>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 ****************************************************************************/

#pragma once

#include "ogrsf_frmts.h"
#include "cpl_string.h"
#include "dl_dxf.h"
#include <vector>

/************************************************************************/
/*							    OGRDXFLayer			                    */
/************************************************************************/

class OGRDXFDataSource;

class OGRDXFLayer : public OGRLayer
{
protected:
	OGRFeatureDefn     *poFeatureDefn;
	OGRDXFDataSource    *poDS;
	char              **papszOptions;
	std::vector<OGRFeature*> m_pFeatures;
	int pos;
public:
	OGRErr       WriteBlocks( DL_Dxf* dxf, DL_WriterA* dw, int pos );
	OGRErr       WriteEntities( DL_Dxf* dxf, DL_WriterA* dw, int pos );
	OGRErr       WriteEntity( DL_Dxf* dxf, DL_WriterA* dw,OGRGeometry * poGeom, const char* lpszLayerName );
	OGRDXFLayer( const char *pszLayerName, char **papszOptions, OGRDXFDataSource *poDS );
    virtual	~OGRDXFLayer();
    virtual void	ResetReading();
    virtual OGRFeature *GetNextFeature();
    OGRFeatureDefn	*GetLayerDefn() { return poFeatureDefn; }
    virtual int         TestCapability( const char *pszCap );
    virtual OGRErr      CreateField( OGRFieldDefn *poField, int bApproxOK = TRUE );
    virtual OGRErr      CreateFeature( OGRFeature *poFeature );
};

/************************************************************************/
/*							  OGRDXFDataSource		                    */
/************************************************************************/

class OGRDXFDataSource : public OGRDataSource
{
private:
    friend class OGRDXFLayer;

    OGRDXFLayer	**papoLayers;
    int	nLayers;
    CPLString	osFilename;
    CPLString	osOutClass;
    //char	**papszOptions;
    OGREnvelope         sExtent;
    void                ExtendExtent( OGRGeometry * );

  public:
	OGRDXFDataSource( const char *pszOutClass );
	~OGRDXFDataSource();
	int	Create( const char *pszFilename, char **papszOptions );
	const char	*GetName(){ return osFilename; };
	int	GetLayerCount() { return nLayers; }
	OGRLayer	*GetLayer( int iLayer );
	virtual OGRLayer	*CreateLayer( const char *pszLayerName,	OGRSpatialReference * = NULL, 
		OGRwkbGeometryType = wkbUnknown, char ** = NULL );
	int	TestCapability( const char * );
private:
	DL_Dxf dxf;
	DL_WriterA *dw;
	char* preAll;
	char* preNum;
	DL_Codes::version exportVersion;
};

/************************************************************************/
/*                             OGRDXFDriver                              */
/************************************************************************/

class OGRDXFDriver : public OGRSFDriver
{
private:
	CPLString    osOutClass;
public:
	OGRDXFDriver( const char *pszName ){ osOutClass = pszName; };
	virtual ~OGRDXFDriver(){};
	virtual const char *GetName(){ return osOutClass; };
	virtual OGRDataSource *Open( const char * pszFilename, int bUpdate );
	virtual OGRDataSource *CreateDataSource( const char *pszFilename, char **papszOptions = NULL );
	virtual OGRErr DeleteDataSource( const char *pszName ){return OGRERR_NONE;};
	virtual int TestCapability( const char * pszCap );
};




